@@ -13,12 +13,14 @@ from vinca.lib import filter
 from vinca.generators import generate, GENERATORS_DICT
 
 # TODO: intelligent scrollback for reviewers and editors
+# how? intelligent parameterization
 # TODO: make functions specify the parameters they need and 
 # feed them this explicitly á la FILTER
 
 TODAY = datetime.date.today()
 
-COLORIZE = sys.stdout.isatty()
+PIPE_IN = not sys.stdin.isatty()
+PIPE_OUT = not sys.stdout.isatty()
 QUIT_KEYS = ['q', readchar.key.ESC, '\n', '\r']
 
 vinca_path = Path(__file__).parent 
@@ -94,6 +96,7 @@ def statistics(args):
 		lines = 3
 	if args.scrollback:
 		ansi.up_line(lines)
+		ansi.hide_cursor()
 
 @subcommand(CARD,alias='e')
 def edit(args):
@@ -155,43 +158,56 @@ def edit_tags(args):
 		tags = ' '.join(args.card.tags)
 		rl.set_startup_hook(lambda: rl.insert_text(tags))
 		args.card.tags = input('tags: ').split()
-		if args.scrollback:
-			ansi.up_line()
+		lines = 1
 		rl.set_startup_hook()
 	elif args.cards:
 		tags_add = input('tags to add: ').split()
 		tags_remove = input('tags to remove: ').split()
+		lines = 2
 		for tag in tags_add:
 			args.tag = tag
 			add_tag(args)
 		for tag in tags_remove:
 			args.tag = tag
 			remove_tag(args)
-
+	
 	rl.set_completer()
 
+	if args.scrollback:
+		ansi.up_line(lines)
+
+
 CMD_DICT = {'r': review, 'R': review,
 	    's': statistics, 'S': statistics,
 	    'x': delete, 'X': delete,
 	    'e': edit, 'E': edit_metadata,
 	    't': edit_tags, 'T': edit_tags}
-@subcommand(CARDS, arg('-t','--default_tags', nargs='+', default=[]), alias='b')
+@subcommand(arg('--cards',nargs='*',type=card_type,default=filter.filter(ALL_CARDS)), arg('-t','--default_tags', nargs='+', default=[]), alias='b')
 def browse(args, reviewing = False):
+	''' Use J and K to move up and down. In general a lowercase letter will act on the selected card while a capital letter will act upon all cards. Use the following hotkeys in browse mode: r, x, s, e, t'''
 	args.scrollback = True
 	cards = args.cards
+	if not cards:
+		print('no cards')
+		return
+	FRAME_WIDTH = 10  # TODO better width?
+	STATUS_BAR = len(cards) > FRAME_WIDTH
 	# TODO max frame of ten cards
 	n = len(cards); sel = 0
 
 	def quit():
 		ansi.show_cursor()
-		ansi.line_wrap_on()
 		ansi.clear_to_end()
 		exit()
-	def draw_browser():
+	def draw_browser(frame):
 		ansi.hide_cursor()
 		ansi.line_wrap_off()
 
-		for i, card in enumerate(cards):
+		if STATUS_BAR:
+			ansi.light()
+			print(f'{sel+1} of {len(cards)}')
+			ansi.reset()
+		for i, card in enumerate(cards[frame:frame+FRAME_WIDTH], start=frame):
 			if card.due_as_of(args.review_date):
 				ansi.bold()
 				ansi.blue()
@@ -202,20 +218,22 @@ def browse(args, reviewing = False):
 				ansi.highlight()
 			print(card)
 			ansi.reset()
+		ansi.line_wrap_on()
 
-	def redraw_browser():
-		ansi.up_line(n)
+	def redraw_browser(frame):
+		ansi.up_line(min(n, 11))
 		ansi.clear_to_end()
-		draw_browser()
+		draw_browser(frame)
 		
-	draw_browser()
+	frame = 0
+	draw_browser(0)
 	while True:
 
 		k = 'R' if reviewing else readchar.readchar()
 
-		sel += k=='j'
-		sel -= k=='k'
-		sel %= n
+		sel += k=='j' and sel < len(cards)-1
+		sel -= k=='k' and sel > 0
+
 
 		if k in QUIT_KEYS:
 				quit()
@@ -235,19 +253,22 @@ def browse(args, reviewing = False):
 			new_cards = generate(args)
 			cards[sel:sel] = new_cards
 			n = len(cards)
-			ansi.down_line(len(new_cards))
+			if n < FRAME_WIDTH:
+				ansi.down_line(len(new_cards))  # TODO better soln
 			ansi.hide_cursor()
 
+		frame += (frame + FRAME_WIDTH == sel)  # scroll down if we are off the screen
+		frame -= (frame - 1 == sel) # scroll up if we are off the screen
 		if k not in ('s','S'):
-			redraw_browser()
+			redraw_browser(frame)
 			
 
 @subcommand(arg('pattern',nargs='?',default=''),
 	arg('-v','--invert',action='store_true'),
 	arg('-i','--id_only',action='store_true'),
 	arg('--cards',nargs='*',type=card_type, default=ALL_CARDS),
-	arg('--tags_include',nargs='*', metavar='TAGS'),
-	arg('--tags_exclude',nargs='*', metavar='TAGS'),
+	arg('--tags_include',nargs='*', metavar='TAGS', default=[]),
+	arg('--tags_exclude',nargs='*', metavar='TAGS', default=[]),
 	arg('--create_date_min',type=date_type, metavar='DATE'),
 	arg('--create_date_max',type=date_type, metavar='DATE'),
 	arg('--seen_date_min',type=date_type, metavar='DATE'),
@@ -266,8 +287,7 @@ def browse(args, reviewing = False):
 	alias='f', name='filter')
 def display_filter(args):
 	''' Filter a set of cards using a variety of parameters.
-	Piping the output of this \033[32m command to another vinca command will
-	provide a list of cards.'''
+	It is often helpful to pipe this command to another one.'''
 
 	# get filter parameters as a list of strings
 	filter_kwargs = inspect.getfullargspec(filter.filter).args[1:]
@@ -276,26 +296,32 @@ def display_filter(args):
 	matches = filter.filter(args.cards,
 		# feed the keyword args editor=args.editor, due=args.due, 
 		**{param : getattr(args, param) for param in filter_kwargs})
-	for card in matches:
-		if args.id_only:
-			print(card.id)
-			continue
-		if COLORIZE:
-			if card.due_as_of(TODAY):
-				ansi.bold()
-				ansi.blue()
-			if card.deleted:
-				ansi.crossout()
-				ansi.red()
+	if PIPE_OUT:
+		print(*[card.id for card in matches], sep='\n', end='')
+		exit()
+	if len(matches) > 10:
+		ansi.light()
+		print(f'10 of {len(matches)}')
+	ansi.line_wrap_off()
+	for card in matches[:10]:
+		if card.due_as_of(TODAY):
+			ansi.bold(); ansi.blue()
+		if card.deleted:
+			ansi.crossout(); ansi.red()
 		print(card.id, card, sep='\t')
-		if COLORIZE:
-			ansi.reset()
+		ansi.reset()
 	
 @subcommand()
 def purge(args):
 	''' Permanently delete all cards marked for deletion. '''
-	for card in filter.filter(ALL_CARDS, deleted_only=True):
-		rmtree(card.path)
+	deleted_cards = filter.filter(ALL_CARDS, deleted_only=True)
+	if not deleted_cards:
+		print('no cards are marked for deletion.')
+		return
+	print(f'delete {len(deleted_cards)} cards? (y/n)')
+	if (confirmation := readchar.readchar()) == 'y':
+		for card in deleted_cards:
+			rmtree(card.path)
 
 @subcommand(arg('backup_path',type=Path), CARDS, name='export')
 def exp(args):
@@ -320,11 +346,12 @@ for alias, generator in GENERATORS_DICT.items():
 
 # parse the command line arguments
 parser.set_defaults(cards = [], func = lambda args: parser.print_help(), review_date = TODAY, scrollback = False, card = None)
+# simplifiy the help message
 parser._action_groups[0].title = 'commands'
 parser._action_groups.pop()
 args = parser.parse_args()
 # accept a file of newline separated card ids
-if not sys.stdin.isatty():
+if PIPE_IN:
 	ids = [int(line.strip().split()[0]) for line in sys.stdin]
 	args.cards = [Card(id) for id in ids]
 	sys.stdin = open('/dev/tty')  
