@@ -17,8 +17,10 @@ def filter(cards, pattern='',
 	# compile the regex pattern for faster searching
 	p = re.compile(f'({pattern})')  # wrap in parens to create regex group \1
 
-	f = lambda card: (((not tags_include or tags_include & card.tags) and
-			(not tags_exclude or not tags_exclude & card.tags) and
+	tags_include, tags_exclude = set(tags_include), set(tags_exclude)
+
+	f = lambda card: (((not tags_include or bool(tags_include & set(card.tags))) and
+			(not tags_exclude or not bool(tags_exclude & set(card.tags))) and
 			(not create_date_min or create_date_min <= card.create_date) and
 			(not create_date_max or create_date_max >= card.create_date) and 
 			(not seen_date_min or seen_date_min <= card.seen_date) and
